@@ -25,7 +25,7 @@
 #define DYNSIZE 1000
 #define STACKSIZE 400000
 #define REMKSIZE 3000000
-#define BUFSIZE 256
+#define BUFSIZE 131072
 #define STRSIZE 10000
 #define SHORT_STRSIZE 32
 #define HASHTBSIZE 137
@@ -34,12 +34,7 @@
 #define CTRLSTK 200
 #define BACKSIZE 30
 #define EISL_PATH_MAX 256
-#if defined(__TINYC__) && defined(__riscv)
-/* TCC in RISC-V cannot handle a larger number */
-#define COL_SIZE 51
-#else
 #define COL_SIZE 255
-#endif
 #define NUM_HISTORY 10
 #define BIGSIZE 20000000
 #define NTTBASE 1000
@@ -89,7 +84,7 @@
 
 
 
-static const float VERSION = 5.43;
+static const float VERSION = 5.53;
 static const int FREESIZE = 1000000;
 static const int SYMSIZE = 256;
 static const int CHARSIZE = 7;	// unicode char. add \0 to tail
@@ -100,12 +95,8 @@ static const int FEND = 6;
 static const int BIGNUM_BASE = 1000000000;
 static const int FAILSE = -1000000000;
 static const int BIGNUM_WORK = BIGSIZE * 5 / 10;	// from 50% to 90% of bigcell area is working area.
-// TCC does not have support for "static conts" as compile time constant
-#ifdef __TINYC__
-#define BIGNUM_PARMA  (BIGSIZE * 9 / 10)
-#else
 static const int BIGNUM_PARMA = BIGSIZE * 9 / 10;	//from 90% to 100% of bigcell area is parmanent area
-#endif
+
 
 typedef enum __packed { EMP, INTN, FLTN, LONGN, BIGN, VEC, ARR, CHR, STR,
 	SYM,
@@ -568,7 +559,7 @@ extern bool repl_flag;
 extern bool org_repl_flag;
 #endif
 extern bool option_flag;
-extern volatile sig_atomic_t exit_flag;
+extern volatile sig_atomic_t ctrl_c_flag;
 extern bool greeting_flag;
 extern bool script_flag;
 extern bool handling_resource_err;
@@ -579,10 +570,11 @@ extern bool parallel_flag;
 extern bool parallel_exit_flag;
 extern bool process_flag;
 extern bool thread_flag;
-extern bool network_flag;
+extern bool parent_flag;
+extern bool child_flag;
 extern bool connect_flag;
 extern bool receiver_exit_flag;
-extern bool child_busy_flag;
+extern bool shutdown_flag;
 
 // try function
 extern bool try_flag;
@@ -658,13 +650,29 @@ extern int child_signal[PROCSIZE];
 extern int child_signal1[PROCSIZE];
 
 /* distributed parallel */
-#define PORT 5000 
+#define PORT 5100 
+extern pthread_mutex_t mutex2;
+extern pthread_cond_t md_cond;
+extern int child_buffer_ready;
 extern int sockfd[PARASIZE];
+extern int parent_sockfd[2];
+extern int child_sockfd[PARASIZE];
 extern socklen_t parent_len;
 extern struct sockaddr_in parent_addr, child_addr[PARASIZE];
 extern int child_num;
+extern int child_id;
+extern int child_pt;
 extern pthread_t receiver_thread;
+extern pthread_t preceiver_thread[PARASIZE];
+extern pthread_t creceiver_thread;
 extern int child_result[PARASIZE];
+extern char parent_buffer[BUFSIZE][PARASIZE];
+extern char child_buffer[BUFSIZE];
+extern char transfer[BUFSIZE];
+extern char input_buffer[BUFSIZE];
+extern char output_buffer[BUFSIZE];
+extern int dp_trace;
+
 
 /* -----TCPIP for server----------------*/
 extern socklen_t server_len;
@@ -823,6 +831,7 @@ int copy(int x);
 int count_args(int ls);
 int create_array(int x);
 int create_list(int x, int y);
+int clear_parent_buffer(int n);
 int dec_token(char buf[]);
 int divide(int arg1, int arg2);
 int duplicate_list_p(int ls);
@@ -910,16 +919,19 @@ int f_domain_error_expected_class(int arglist, int th);
 int f_domain_error_object(int arglist, int th);
 int f_dp_create(int arglist, int th);
 int f_dp_close(int arglist, int th);
-int f_dp_let(int arglist, int th);
+int f_dp_halt(int arglist, int th);
 int f_dp_eval(int arglist, int th);
 int f_dp_transfer(int arglist, int th);
-int f_dp_receive(int arglist, int th);
 int f_dp_load(int arglist, int th);
 int f_dp_compile(int arglist, int th);
 int f_dp_call(int arglist, int th);
 int f_dp_exec(int arglist, int th);
-int f_dp_part(int arglist, int th);
-int f_dp_report(int arglist, int th);
+int f_dp_and(int arglist, int th);
+int f_dp_or(int arglist, int th);
+int f_dp_setid(int arglist, int th);
+int f_dp_senderr(int arglist, int th);
+int f_dp_trace(int arglist, int th);
+int f_dp_untrace(int arglist, int th);
 int f_dummyp(int arglist, int th);
 int f_dynamic(int arglist, int th);
 int f_dynamic_let(int arglist, int th);
@@ -1011,9 +1023,8 @@ int f_mp_create(int arglist, int th);
 int f_mp_exec(int arglist, int th);
 int f_mp_close(int arglist, int th);
 int f_mp_call(int arglist, int th);
-int f_mp_let(int arglist, int th);
-int f_mp_part(int arglist, int th);
-int f_mp_report(int arglist, int th);
+int f_mp_and(int arglist, int th);
+int f_mp_or(int arglist, int th);
 int f_mp_eval(int arglist, int th);
 int f_mt_call(int arglist, int th);
 int f_mt_lock(int arglist, int th);
@@ -1298,9 +1309,6 @@ int read_from_pipe(int n);
 int read_from_pipe_part(int n);
 int receive_from_child(int n);
 int receive_from_parent(void);
-int receive_from_child_part(int n, int opt);
-int receive_from_child_part1(int n, int opt);
-int receive_from_child_part2(int n);
 int send_to_child(int n, int x);
 int sexp_to_str(int x);
 int str_to_sexp(int x);
@@ -1346,6 +1354,9 @@ int vector_length(int x);
 int vector_ref(int v, int n);
 int vector_to_list(int x);
 int vectorp(int x);
+int wait_all(int m);
+int wait_and(int m);
+int wait_or(int m);
 int wait_para(void);
 int write_to_pipe(int n, int x);
 int zerop(int addr);
@@ -1398,11 +1409,14 @@ void init_syntax(void);
 void init_tcpip(void);
 void init_parent(void);
 void init_child(int n, int x);
+void init_preceiver(int n);
+void init_creceiver(void);
 void insert_method(int x, int func);
 void insert_str(char ch, char buf[]);
 void load(int x, int th);
 void mark_cell(int addr);
 void print(int addr);
+void print_ascii(char *str);
 void print_big(int x);
 void print_arr(int x);
 void print_char(int addr);
@@ -1432,6 +1446,9 @@ void set_val(int sym, int val, int ls);
 void signal_handler_c(int signo);
 void signal_handler_child(int sig, siginfo_t *siginfo, void *context);
 void send_to_parent(int x);
+void send_to_parent_control(int code);
+void send_to_child_control(int n, int code);
+void send_to_child_control_without_0x16(int n, int code);
 void unbind(int th);
 void unreadc(char c);
 void ucs4_to_utf8(int n, char *p);
@@ -1462,6 +1479,17 @@ int f_delay(int arglist, int th);
 int f_delay_microseconds(int arglist, int th);
 #endif
 
+#ifdef __rpigpio__
+int f_gpio_init(int arglist, int th);
+int f_gpio_set_mode(int arglist, int th);
+int f_gpio_write(int arglist, int th);
+int f_gpio_read(int arglist, int th);
+int f_gpio_event_request(int arglist, int th);
+int f_gpio_event_wait(int arglist, int th);
+int f_gpio_event_read(int arglist, int th);
+int f_gpio_close(int arglist, int th);
+#endif
+
 void debugger(int th);
 void ntt_test();
 
