@@ -13,6 +13,7 @@
  * syntax.c      syntax functions
  * extension.c   extended functions (non ISLisp standard)
  * gbc.c         garbage collenction
+ * parallel.c    parallel functions (non ISLisp standard)
  * edlis.c       Edlis simple CUI editor
  *
  * <cell type summary>
@@ -41,15 +42,12 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#include "compat/term_stubs.h"
 #include "eisl.h"
 #include "mem.h"
 #include "fmt.h"
 #include "except.h"
 #include "str.h"
 #include "long.h"
-#include "compat/eiffel_stubs.h"
-#include "compat/curses_stubs.h"
 
 
 /* pointer */
@@ -128,7 +126,7 @@ int column;
 char buffer[COL_SIZE + 1][NUM_HISTORY];
 char buffer1[COL_SIZE + 1];
 char buffer2[COL_SIZE + 1] = { 0 };	//for read_stdin()
-char buffer3[STRSIZE] = { 0 };	//for Multi-Process and Distributed TCP/IP
+char buffer3[STRSIZE] = { 0 };	//for Multi-Process 
 
 /* heap ,stack and bignum */
 cell heap[CELLSIZE];
@@ -161,13 +159,11 @@ bool redef_flag = false;	/* true=redefine-class,false=not-redefine */
 bool start_flag = true;		/* true=line-start,false=not-line-start */
 bool back_flag = true;		/* for backtrace,true=on,false=off */
 bool ignore_topchk = false;	/* for FAST-compiler true=ignore,false=normal */
-#ifndef WITHOUT_CURSES
 bool repl_flag = true;		/* for REPL read_line true=on,false=off */
 bool org_repl_flag = true;	/* original val for restore */
-#endif
 bool option_flag = false;	/* while handling command line option it is true, else false */
-volatile sig_atomic_t exit_flag = 0;	/* true= ctrl+C */
-bool greeting_flag = true;	/* for (quit) */
+volatile sig_atomic_t ctrl_c_flag = 0;	/* true= ctrl+C */
+bool greeting_flag = true;	/* for greeting */
 bool script_flag = false;	/* for -s option */
 bool handling_resource_err = false;	/* stop infinite recursion */
 bool looking_for_shebang = false;	/* skip over #! */
@@ -177,10 +173,11 @@ bool parallel_flag = false;	/* while executing parallel */
 bool parallel_exit_flag = false;	/* To exit parallel threads */
 bool process_flag = false;	/* when invoke as child process, flag is true */
 bool thread_flag = false;	/* when invoke as multi thread, flag is true */
-bool network_flag = false;	/* when invoke as network child, flag is true */
+bool parent_flag = false;	/* when invoke as parent, flag is true */
+bool child_flag = false;	/* when invoke as network child, flag is true */
 bool connect_flag = false;	/* when child listen, connect_flag is true */
-bool receiver_exit_flag = false;	/* TO exit child TCP/IP receiver */
-bool child_busy_flag = false;	/* while evalating in child, child_buzy_flag is true */
+bool receiver_exit_flag = false;	/* to exit child TCP/IP receiver */
+bool shutdown_flag = false;	/* shutdown OS on child */
 /* try function (try time s-exp binary) */
 bool try_flag;			/* true or false */
 double try_timer;		/* limit timer */
@@ -252,12 +249,25 @@ int child_signal[PROCSIZE];
 int child_signal1[PROCSIZE];
 
 /* -----distributed parallel & TCPIP------*/
-int sockfd[PARASIZE];
+pthread_mutex_t mutex2;
+pthread_cond_t md_cond;
+int child_buffer_ready;
+int parent_sockfd[2];
+int child_sockfd[PARASIZE];
 socklen_t parent_len;
 struct sockaddr_in parent_addr, child_addr[PARASIZE];
 int child_num;
+int child_id;
 pthread_t receiver_thread;
+pthread_t preceiver_thread[PARASIZE];
+pthread_t creceiver_thread;
 int child_result[PARASIZE];
+char parent_buffer[BUFSIZE][PARASIZE];
+char child_buffer[BUFSIZE];
+char transfer[BUFSIZE];
+char input_buffer[BUFSIZE];
+char output_buffer[BUFSIZE];
+int dp_trace = 0;
 
 /* -----TCPIP for server----------------*/
 socklen_t server_len;
@@ -314,14 +324,14 @@ static inline void maybe_greet(void)
 {
     if (greeting_flag)
 	Fmt_print("Easy-ISLisp Ver%1.2f\n", VERSION);
+
+    greeting_flag = false;
 }
 
 static inline void disable_repl_flag(void)
 {
-#ifndef WITHOUT_CURSES
     repl_flag = false;
     org_repl_flag = false;
-#endif
 }
 
 int main(int argc, char *argv[])
@@ -357,11 +367,7 @@ int main(int argc, char *argv[])
     memset(&child_action, 0, sizeof(child_action));
     child_action.sa_sigaction = &signal_handler_child;
     child_action.sa_flags = SA_SIGINFO;
-#ifdef __linux__
     sigaction(SIGRTMIN, &child_action, NULL);
-#else
-    sigaction(SIGUSR1, &child_action, NULL);
-#endif
     if (setenv("EASY_ISLISP", STRQUOTE(SHAREDIR), /* overwrite = */ 0) ==
 	-1) {
 	perror("setenv");
@@ -422,9 +428,9 @@ int main(int argc, char *argv[])
 		break;
 	    case 'n':
 		puts("EISL runs with network mode.");
-		network_flag = true;
+		child_flag = true;
 		init_parent();
-		init_receiver();
+		init_creceiver();
 		str = library_file("compiler.lsp");
 		f_load(list1(make_str(str)), 0);
 		FREE(str);
@@ -456,11 +462,10 @@ int main(int argc, char *argv[])
     /* REPL */
     volatile bool quit = false;
     do {
-	if (!process_flag)
-	    maybe_greet();
+	maybe_greet();
 	TRY while (1) {
 	    init_pointer();
-	    if (!process_flag && !network_flag) {
+	    if (!process_flag && !child_flag) {
 		fputs("> ", stdout);
 		print(eval(sread(), 0));
 		putchar('\n');
@@ -470,36 +475,30 @@ int main(int argc, char *argv[])
 		fflush(stdout);
 		union sigval value;
 		value.sival_int = (int) process_num;
-#ifdef __linux__
 		sigqueue(getppid(), SIGRTMIN, value);
-#else
-		kill(getppid(), SIGUSR1);
-#endif
 		EXCEPT(Exit_Process);
 		END_TRY;
-	    } else if (network_flag) {
+	    } else if (child_flag) {
+		pthread_mutex_lock(&mutex2);
+		while (!child_buffer_ready) {
+		    pthread_cond_wait(&md_cond, &mutex2);
+		}
+		child_buffer_ready = 0;
+		pthread_mutex_unlock(&mutex2);
 		int exp, res;
 		exp = str_to_sexp(receive_from_parent());
 		printf("receive_from_parent ");
 		print(exp);
 		printf("\n");
 		fflush(stdout);
-		if (equalp(exp, make_int(999))) {
-		    printf("exit_from_network_mode\n");
-		    close_socket();
-		    exit(0);
-		} else {
-		    TRY child_busy_flag = true;
-		    res = eval(exp, 0);
-		    child_busy_flag = false;
-		    printf("send_to_parent ");
-		    print(res);
-		    printf("\n");
-		    fflush(stdout);
-		    send_to_parent(sexp_to_str(res));
-		    EXCEPT(Exit_Network);
-		    END_TRY;
-		}
+		TRY res = eval(exp, 0);
+		printf("send_to_parent ");
+		print(res);
+		printf("\n");
+		fflush(stdout);
+		send_to_parent(sexp_to_str(res));
+		EXCEPT(Exit_Network);
+		END_TRY;
 	    }
 
 	    if (redef_flag)
@@ -510,6 +509,12 @@ int main(int argc, char *argv[])
 	    quit = true;
 	    exit_thread();
 	    close_socket();
+	    if (shutdown_flag) {
+		printf("Shutting down the system...\n");
+		int ret = system("sudo shutdown now");
+		if (ret == -1)
+		    error(SYSTEM_ERR, "dp-halt shatdown", NIL, 0);
+	    }
 	}
 	END_TRY;
     }
@@ -576,7 +581,7 @@ void exit_thread(void)
 
 void signal_handler_c(int signo __unused)
 {
-    exit_flag = 1;
+    ctrl_c_flag = 1;
 }
 
 void signal_handler_child(int sig, siginfo_t * siginfo, void *context)
@@ -628,7 +633,6 @@ int readc(void)
 	c = getc(GET_PORT(input_stream));
 	if (!script_flag && input_stream == standard_input && c == EOF) {
 	    /* ctrl+D and not script-mode quit system */
-	    greeting_flag = false;
 	    putchar('\n');
 	    RAISE(Exit_Interp);
 	} else if (script_flag)
@@ -800,8 +804,7 @@ void get_token(void)
 
 		while (((c = readc()) != EOL) && (c != EOF)
 		       && (pos < stok.bufsize - 1) && (c != SPACE)
-		       && (c != '(')
-		       && (c != ')')) {
+		       && (c != '(') && (c != ')')) {
 		    set_stok_buf(pos++, c);
 		}
 
